@@ -2,7 +2,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -133,7 +133,7 @@ DECLARE_BUILTIN_FUNCTION(ShiftFunction)
 DECLARE_BUILTIN_FUNCTION_R(AbsFunction)
 DECLARE_BUILTIN_FUNCTION(GcdFunction)
 DECLARE_BUILTIN_FUNCTION(LcmFunction)
-DECLARE_BUILTIN_FUNCTION(SignumFunction)
+DECLARE_BUILTIN_FUNCTION_R(SignumFunction)
 DECLARE_BUILTIN_FUNCTION_R(RoundFunction)
 DECLARE_BUILTIN_FUNCTION_R(FloorFunction)
 DECLARE_BUILTIN_FUNCTION_R(CeilFunction)
@@ -156,9 +156,13 @@ DECLARE_BUILTIN_FUNCTION(LDegreeFunction)
 
 DECLARE_BUILTIN_FUNCTION_R(ReFunction)
 DECLARE_BUILTIN_FUNCTION_R(ImFunction)
-DECLARE_BUILTIN_FUNCTION(ArgFunction)
+DECLARE_BUILTIN_FUNCTION_R2(ArgFunction)
 
-DECLARE_BUILTIN_FUNCTION(SqrtFunction)
+DECLARE_BUILTIN_FUNCTION(IntervalFunction)
+
+DECLARE_BUILTIN_FUNCTION_R(SqrtFunction)
+DECLARE_BUILTIN_FUNCTION(CbrtFunction)
+DECLARE_BUILTIN_FUNCTION_R(RootFunction)
 DECLARE_BUILTIN_FUNCTION(SquareFunction)
 
 DECLARE_BUILTIN_FUNCTION(ExpFunction)
@@ -170,7 +174,7 @@ DECLARE_BUILTIN_FUNCTION(LambertWFunction)
 
 DECLARE_BUILTIN_FUNCTION_R2(SinFunction)
 DECLARE_BUILTIN_FUNCTION_R2(CosFunction)
-DECLARE_BUILTIN_FUNCTION(TanFunction)
+DECLARE_BUILTIN_FUNCTION_R2(TanFunction)
 DECLARE_BUILTIN_FUNCTION_R1(AsinFunction)
 DECLARE_BUILTIN_FUNCTION_R1(AcosFunction)
 DECLARE_BUILTIN_FUNCTION_R2(AtanFunction)
@@ -180,11 +184,19 @@ DECLARE_BUILTIN_FUNCTION_R2(TanhFunction)
 DECLARE_BUILTIN_FUNCTION_R2(AsinhFunction)
 DECLARE_BUILTIN_FUNCTION_R1(AcoshFunction)
 DECLARE_BUILTIN_FUNCTION(AtanhFunction)
+DECLARE_BUILTIN_FUNCTION_R1(Atan2Function)
 DECLARE_BUILTIN_FUNCTION(RadiansToDefaultAngleUnitFunction)
+DECLARE_BUILTIN_FUNCTION_R2(SincFunction)
 
 DECLARE_BUILTIN_FUNCTION(ZetaFunction)
 DECLARE_BUILTIN_FUNCTION(GammaFunction)
+DECLARE_BUILTIN_FUNCTION(DigammaFunction)
 DECLARE_BUILTIN_FUNCTION(BetaFunction)
+DECLARE_BUILTIN_FUNCTION(AiryFunction)
+DECLARE_BUILTIN_FUNCTION(BesseljFunction)
+DECLARE_BUILTIN_FUNCTION(BesselyFunction)
+DECLARE_BUILTIN_FUNCTION(ErfFunction)
+DECLARE_BUILTIN_FUNCTION(ErfcFunction)
 
 DECLARE_BUILTIN_FUNCTION(TotalFunction)
 DECLARE_BUILTIN_FUNCTION(PercentileFunction)
@@ -259,6 +271,6 @@ DECLARE_BUILTIN_FUNCTION(IntegrateFunction)
 DECLARE_BUILTIN_FUNCTION(SolveFunction)
 DECLARE_BUILTIN_FUNCTION(SolveMultipleFunction)
 
-DECLARE_BUILTIN_FUNCTION(UncertaintyFunction)
+DECLARE_BUILTIN_FUNCTION(PlotFunction)
 
 #endif
