@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2004-2006, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2004-2006, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -20,9 +20,9 @@
 #include <libxml/xmlmemory.h>
 #include <libxml/parser.h>
 #include <errno.h>
+#include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <glib.h>
 
 #define XML_GET_STRING_FROM_PROP(node, name, str)	value = xmlGetProp(node, (xmlChar*) name); if(value) {str = (char*) value; remove_blank_ends(str); xmlFree(value);} else str = ""; 
 #define XML_GET_STRING_FROM_TEXT(node, str)		value = xmlNodeListGetString(doc, node->xmlChildrenNode, 1); if(value) {str = (char*) value; remove_blank_ends(str); xmlFree(value);} else str = "";
@@ -420,83 +420,77 @@ void DataSet::setDefaultDataFile(string s_file) {
 const string &DataSet::defaultDataFile() const {
 	return sfile;
 }
-	
+#ifdef _WIN32
+#	define FILE_SEPARATOR_CHAR '\\'
+#else
+#	define FILE_SEPARATOR_CHAR '/'
+#endif
 bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 	if(file_name) {
 	} else if(sfile.empty()) {
 		return false;
-	} else if(sfile.find(G_DIR_SEPARATOR_S) != string::npos) {
+	} else if(sfile.find(FILE_SEPARATOR_CHAR) != string::npos) {
 		bool b = loadObjects(file_name, false);
-		size_t i = sfile.find_last_of(G_DIR_SEPARATOR_S);
+		size_t i = sfile.find_last_of(FILE_SEPARATOR_CHAR);
 		if(i != sfile.length() - 1) {
-			gchar *filepath = g_build_filename(getLocalDataDir().c_str(), "definitions", "datasets", sfile.substr(i + 1, sfile.length() - (i + 1)).c_str(), NULL);
-			if(loadObjects(filepath, true)) {
+			string filepath = buildPath(getLocalDataDir(), "definitions", "datasets", sfile.substr(i + 1, sfile.length() - (i + 1)));
+			if(loadObjects(filepath.c_str(), true)) {
 				b = true;
 			} else {
-				gchar *filepath_old = g_build_filename(getOldLocalDir().c_str(), "definitions", "datasets", sfile.substr(i + 1, sfile.length() - (i + 1)).c_str(), NULL);
-				if(loadObjects(filepath_old, true)) {
+#ifndef _WIN32
+				string filepath_old = buildPath(getOldLocalDir(), "definitions", "datasets", sfile.substr(i + 1, sfile.length() - (i + 1)));
+				if(loadObjects(filepath_old.c_str(), true)) {
 					b = true;
-					mkdir(getLocalDataDir().c_str(), S_IRWXU);
-					gchar *dir = g_build_filename(getLocalDataDir().c_str(), "definitions", NULL);
-					mkdir(dir, S_IRWXU);
-					g_free(dir);
-					dir = g_build_filename(getLocalDataDir().c_str(), "definitions", "datasets", NULL);
-					mkdir(dir, S_IRWXU);
-					g_free(dir);
-					move_file(filepath_old, filepath);
-					dir = g_build_filename(getOldLocalDir().c_str(), "definitions", "datasets", NULL);
-					rmdir(dir);
-					g_free(dir);
-					dir = g_build_filename(getOldLocalDir().c_str(), "definitions", NULL);
-					rmdir(dir);
-					g_free(dir);
+					makeDir(getLocalDataDir());
+					makeDir(buildPath(getLocalDataDir(), "definitions"));
+					makeDir(buildPath(getLocalDataDir(), "definitions", "datasets"));
+					move_file(filepath_old.c_str(), filepath.c_str());
+					removeDir(buildPath(getOldLocalDir(), "definitions", "datasets"));
+					removeDir(buildPath(getOldLocalDir(), "definitions"));
 				}
-				g_free(filepath_old);
+#endif
 			}
-			g_free(filepath);
 		}
 		return b;
 	} else {
-		gchar *filepath = g_build_filename(PACKAGE_DATA_DIR, "qalculate", sfile.c_str(), NULL);
-		bool b = loadObjects(filepath, false);
-		g_free(filepath);
-		filepath = g_build_filename(getLocalDataDir().c_str(), "definitions", "datasets", sfile.c_str(), NULL);
-		if(loadObjects(filepath, true)) {
+		bool b = loadObjects(buildPath(getGlobalDefinitionsDir(), sfile).c_str(), false);
+		string filepath = buildPath(getLocalDataDir(), "definitions", "datasets", sfile);
+		if(b && !fileExists(filepath)) return true;
+		if(loadObjects(filepath.c_str(), true)) {
 			b = true;
 		} else {
-			gchar *filepath_old = g_build_filename(getOldLocalDir().c_str(), "definitions", "datasets", sfile.c_str(), NULL);
-			if(loadObjects(filepath_old, true)) {
+#ifndef _WIN32
+			string filepath_old = buildPath(getOldLocalDir(), "definitions", "datasets", sfile);
+			if(loadObjects(filepath_old.c_str(), true)) {
 				b = true;
-				mkdir(getLocalDataDir().c_str(), S_IRWXU);
-				gchar *dir = g_build_filename(getLocalDataDir().c_str(), "definitions", NULL);
-				mkdir(dir, S_IRWXU);
-				g_free(dir);
-				dir = g_build_filename(getLocalDataDir().c_str(), "definitions", "datasets", NULL);
-				mkdir(dir, S_IRWXU);
-				g_free(dir);
-				move_file(filepath_old, filepath);
-				dir = g_build_filename(getOldLocalDir().c_str(), "definitions", "datasets", NULL);
-				rmdir(dir);
-				g_free(dir);
-				dir = g_build_filename(getOldLocalDir().c_str(), "definitions", NULL);
-				rmdir(dir);
-				g_free(dir);
+				makeDir(getLocalDataDir());
+				makeDir(buildPath(getLocalDataDir(), "definitions"));
+				makeDir(buildPath(getLocalDataDir(), "definitions", "datasets"));
+				move_file(filepath_old.c_str(), filepath.c_str());
+				removeDir(buildPath(getOldLocalDir(), "definitions", "datasets"));
+				removeDir(buildPath(getOldLocalDir(), "definitions"));
 			}
-			g_free(filepath_old);
+#endif
 		}
-		g_free(filepath);
 		return b;
 	}
 	xmlDocPtr doc;
 	xmlNodePtr cur, child;
 
 	string locale, lang_tmp;
+#ifdef _WIN32
+	WCHAR wlocale[LOCALE_NAME_MAX_LENGTH];
+	if(LCIDToLocaleName(LOCALE_USER_DEFAULT, wlocale, LOCALE_NAME_MAX_LENGTH, 0) != 0) locale = utf8_encode(wlocale);
+	gsub("-", "_", locale);
+#else
 	char *clocale = setlocale(LC_MESSAGES, "");
-	if(clocale) {
-		locale = clocale;
-		if(locale == "POSIX" || locale == "C") {
-			locale = "";
-		}
+	if(clocale) locale = clocale;
+#endif
+	if(locale == "POSIX" || locale == "C") {
+		locale = "";
+	} else {
+		size_t i = locale.find('.');
+		if(i != string::npos) locale = locale.substr(0, i);
 	}
 
 	string localebase;
@@ -536,16 +530,6 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 		xmlFreeDoc(doc);
 		return false;
 	}
-	/*int version_numbers[] = {0, 0, 0};
-	for(size_t i = 0; i < 3; i++) {
-		size_t dot_i = version.find(".");
-		if(dot_i == string::npos) {
-			version_numbers[i] = s2i(version);
-			break;
-		}
-		version_numbers[i] = s2i(version.substr(0, dot_i));
-		version = version.substr(dot_i + 1, version.length() - (dot_i + 1));
-	}*/
 	DataObject *o = NULL;
 	cur = cur->xmlChildrenNode;
 	string str, str2;
@@ -727,23 +711,19 @@ bool DataSet::loadObjects(const char *file_name, bool is_user_defs) {
 	xmlFreeDoc(doc);
 	b_loaded = true;
 	if(!scopyright.empty()) {
-		CALCULATOR->message(MESSAGE_INFORMATION, "%s", scopyright.c_str(), NULL);
+		CALCULATOR->message(MESSAGE_INFORMATION, scopyright.c_str(), NULL);
 	}
 	return true;
 }
 int DataSet::saveObjects(const char *file_name, bool save_global) {
 	string str, filename;
 	if(!save_global && !file_name) {
-		mkdir(getLocalDataDir().c_str(), S_IRWXU);
-		gchar *gstr = g_build_filename(getLocalDataDir().c_str(), "definitions", NULL);		
-		mkdir(gstr, S_IRWXU);
-		gchar *gstr2 = g_build_filename(gstr, "datasets", NULL);
-		mkdir(filename.c_str(), S_IRWXU);
-		gchar *gstr3 = g_build_filename(gstr, sfile.c_str(), NULL);
-		filename = gstr3;
-		g_free(gstr);
-		g_free(gstr2);
-		g_free(gstr3);
+		makeDir(getLocalDataDir());
+		filename = buildPath(getLocalDataDir(), "definitions");
+		makeDir(filename);
+		filename = buildPath(filename, "datasets");
+		makeDir(filename);
+		filename = buildPath(filename, sfile);
 	} else {
 		filename = file_name;
 	}
