@@ -1,7 +1,7 @@
 /*
     Qalculate    
 
-    Copyright (C) 2008  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2008  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -19,6 +19,8 @@
 #include "Number.h"
 #include "Unit.h"
 
+#include <limits.h>
+
 #if HAVE_UNORDERED_MAP
 #	include <unordered_map>
 #elif 	defined(__GNUC__)
@@ -113,6 +115,16 @@ int MathFunction::subtype() const {
 	return SUBTYPE_FUNCTION;
 }
 
+string MathFunction::example(bool raw_format, string name_string) const {
+	if(raw_format) return sexample;
+	string str = sexample;
+	gsub("$name", name_string.empty() ? name() : name_string, str);
+	return CALCULATOR->localizeExpression(str);
+}
+void MathFunction::setExample(string new_example) {
+	sexample = new_example;
+}
+
 /*int MathFunction::countArgOccurence(size_t arg_) {
 	if((int) arg_ > argc && max_argc < 0) {
 		arg_ = argc + 1;
@@ -149,7 +161,7 @@ bool MathFunction::testCondition(const MathStructure &vargs) {
 	eo.approximation = APPROXIMATION_APPROXIMATE;
 	mstruct.eval(eo);
 	if(!mstruct.isNumber() || !mstruct.number().isPositive()) {
-		if(CALCULATOR->showArgumentErrors()) {
+		if(CALCULATOR->showArgumentErrors() && !CALCULATOR->aborted()) {
 			CALCULATOR->error(true, _("%s() requires that %s"), name().c_str(), printCondition().c_str(), NULL);
 		}
 		return false;
@@ -220,7 +232,7 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 	bool last_is_vctr = false, vctr_started = false;
 	if(maxargs() > 0) {
 		arg = getArgumentDefinition(maxargs());
-		last_is_vctr = arg && arg->type() == ARGUMENT_TYPE_VECTOR;
+		last_is_vctr = (arg && arg->type() == ARGUMENT_TYPE_VECTOR);
 	}
 	for(size_t str_index = 0; str_index < str.length(); str_index++) {
 		switch(str[str_index]) {
@@ -287,7 +299,9 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 						}
 					} else if(last_is_vctr) {
 						if(!vctr_started) {
-							vargs[vargs.size() - 1].transform(STRUCT_VECTOR);
+							if(!vargs[vargs.size() - 1].isVector() || vargs[vargs.size() - 1].size() != 1) {
+								vargs[vargs.size() - 1].transform(STRUCT_VECTOR);
+							}
 							vctr_started = true;
 						}
 						stmp = str.substr(start_pos, str_index - start_pos);
@@ -344,7 +358,9 @@ int MathFunction::args(const string &argstr, MathStructure &vargs, const ParseOp
 			}
 		} else if(last_is_vctr) {
 			if(!vctr_started) {
-				vargs[vargs.size() - 1].transform(STRUCT_VECTOR);
+				if(!vargs[vargs.size() - 1].isVector() || vargs[vargs.size() - 1].size() != 1) {
+					vargs[vargs.size() - 1].transform(STRUCT_VECTOR);
+				}
 				vctr_started = true;
 			}
 			stmp = str.substr(start_pos, str.length() - start_pos);
@@ -414,6 +430,7 @@ void MathFunction::setArgumentDefinition(size_t index, Argument *argdef) {
 	if(index > last_argdef_index) {
 		last_argdef_index = index;
 	}
+	argdef->setIsLastArgument((int) index == maxargs());
 	setChanged(true);
 }
 bool MathFunction::testArgumentCount(int itmp) {
@@ -522,7 +539,7 @@ MathStructure MathFunction::calculate(MathStructure &vargs, const EvaluationOpti
 			}
 			return createFunctionMathStructureFromVArgs(vargs);
 		}
-		if(precision() > 0 && (precision() < mstruct.precision() || mstruct.precision() < 1)) mstruct.setPrecision(precision(), true);
+		if(precision() >= 0 && (precision() < mstruct.precision() || mstruct.precision() < 1)) mstruct.setPrecision(precision(), true);
 		if(isApproximate()) mstruct.setApproximate(true, true);
 		return mstruct;
 	} else {
@@ -688,6 +705,7 @@ bool MathFunction::representsOdd(const MathStructure&, bool) const {return false
 bool MathFunction::representsUndefined(const MathStructure&) const {return false;}
 bool MathFunction::representsBoolean(const MathStructure&) const {return false;}
 bool MathFunction::representsNonMatrix(const MathStructure &vargs) const {return representsNumber(vargs, true);}
+bool MathFunction::representsScalar(const MathStructure &vargs) const {return representsNonMatrix(vargs);}
 
 UserFunction::UserFunction(string cat_, string name_, string formula_, bool is_local, int argc_, string title_, string descr_, int max_argc_, bool is_active) : MathFunction(name_, argc_, max_argc_, cat_, title_, descr_, is_active) {
 	b_local = is_local;
@@ -904,11 +922,11 @@ int UserFunction::calculate(MathStructure &mstruct, const MathStructure &vargs,
 		for(size_t i = 0; i < v_id.size(); i++) {
 			CALCULATOR->delId(v_id[i]);
 		}
-		if(precision() > 0) mstruct.setPrecision(precision(), true);
+		if(precision() >= 0) mstruct.setPrecision(precision(), true);
 		if(isApproximate()) mstruct.setApproximate(true, true);
 	} else {
 		CALCULATOR->parse(&mstruct, sformula_calc, po);
-		if(precision() > 0) mstruct.setPrecision(precision(), true);
+		if(precision() >= 0) mstruct.setPrecision(precision(), true);
 		if(isApproximate()) mstruct.setApproximate(true, true);
 	}
 	return 1;
@@ -1123,6 +1141,7 @@ Argument::Argument(string name_, bool does_test, bool does_error) {
 	b_text = false;
 	b_error = does_error;
 	b_rational = false;
+	b_last = false;
 }
 Argument::Argument(const Argument *arg) {
 	b_text = false;
@@ -1171,6 +1190,7 @@ void Argument::set(const Argument *arg) {
 	b_test = arg->tests();
 	b_matrix = arg->matrixAllowed();
 	b_rational = arg->rationalPolynomial();
+	b_last = arg->isLastArgument();
 }
 bool Argument::test(MathStructure &value, int index, MathFunction *f, const EvaluationOptions &eo) const {
 	if(!b_test) {
@@ -1382,6 +1402,8 @@ int Argument::type() const {
 }
 bool Argument::matrixAllowed() const {return b_matrix;}
 void Argument::setMatrixAllowed(bool allow_matrix) {b_matrix = allow_matrix;}
+bool Argument::isLastArgument() const {return b_last;}
+void Argument::setIsLastArgument(bool is_last) {b_last = is_last;}
 bool Argument::rationalPolynomial() const {return b_rational;}
 void Argument::setRationalPolynomial(bool rational_polynomial) {b_rational = rational_polynomial;}
 
@@ -1488,9 +1510,13 @@ bool NumberArgument::subtest(MathStructure &value, const EvaluationOptions &eo)
 	if(!value.isNumber()) {
 		value.eval(eo);
 	}
-	if(!value.isNumber() || (b_rational_number && !value.number().isRational()) || (!b_complex && value.number().isComplex())) {
+	if(!value.isNumber() || (b_rational_number && !value.number().isRational())) {
 		return false;
 	}
+	if(!b_complex && value.number().hasImaginaryPart()) {
+		if(!value.number().imaginaryPartIsNonZero()) value.number().clearImaginary();
+		else return false;
+	}
 	if(fmin) {
 		ComparisonResult cmpr = fmin->compare(value.number());
 		if(!(cmpr == COMPARISON_RESULT_GREATER || (b_incl_min && COMPARISON_IS_EQUAL_OR_GREATER(cmpr)))) {
@@ -1574,9 +1600,10 @@ string NumberArgument::subprintlong() const {
 	return str;
 }
 
-IntegerArgument::IntegerArgument(string name_, ArgumentMinMaxPreDefinition minmax, bool does_test, bool does_error) : Argument(name_, does_test, does_error) {
+IntegerArgument::IntegerArgument(string name_, ArgumentMinMaxPreDefinition minmax, bool does_test, bool does_error, IntegerType integer_type) : Argument(name_, does_test, does_error) {
 	imin = NULL;
 	imax = NULL;
+	i_inttype = integer_type;
 	switch(minmax) {
 		case ARGUMENT_MIN_MAX_POSITIVE: {
 			imin = new Number(1, 1);
@@ -1600,6 +1627,7 @@ IntegerArgument::IntegerArgument(string name_, ArgumentMinMaxPreDefinition minma
 IntegerArgument::IntegerArgument(const IntegerArgument *arg) {
 	imin = NULL;
 	imax = NULL;
+	i_inttype = INTEGER_TYPE_NONE;
 	set(arg);
 }
 IntegerArgument::~IntegerArgument() {
@@ -1610,7 +1638,9 @@ IntegerArgument::~IntegerArgument() {
 		delete imax;
 	}
 }
-	
+
+IntegerType IntegerArgument::integerType() const {return i_inttype;}
+void IntegerArgument::setIntegerType(IntegerType integer_type) {i_inttype = integer_type;}
 void IntegerArgument::setMin(const Number *nmin) {
 	if(!nmin) {
 		if(imin) {
@@ -1647,7 +1677,7 @@ bool IntegerArgument::subtest(MathStructure &value, const EvaluationOptions &eo)
 	if(!value.isNumber()) {
 		value.eval(eo);
 	}
-	if(!value.isNumber() || !value.number().isInteger()) {
+	if(!value.isNumber() || !value.number().isInteger(i_inttype)) {
 		return false;
 	}
 	if(imin) {
@@ -1686,7 +1716,8 @@ void IntegerArgument::set(const Argument *arg) {
 		}
 		if(iarg->max()) {
 			imax = new Number(*iarg->max());
-		}		
+		}
+		i_inttype = iarg->integerType();
 	}
 	Argument::set(arg);
 }
@@ -1700,9 +1731,21 @@ string IntegerArgument::subprintlong() const {
 		str += _(">=");
 		str += " ";
 		str += imin->print();
+	} else if(i_inttype != INTEGER_TYPE_NONE) {
+		str += " ";
+		str += _(">=");
+		str += " ";
+		switch(i_inttype) {
+			case INTEGER_TYPE_SIZE: {}
+			case INTEGER_TYPE_UINT: {str += "0"; break;}
+			case INTEGER_TYPE_SINT: {str += i2s(INT_MIN); break;}
+			case INTEGER_TYPE_ULONG: {str += "0"; break;}
+			case INTEGER_TYPE_SLONG: {str += i2s(LONG_MIN); break;}
+			default: {}
+		}
 	}
 	if(imax) {
-		if(imin) {
+		if(imin || i_inttype != INTEGER_TYPE_NONE) {
 			str += " ";
 			str += _("and");
 		}
@@ -1710,6 +1753,20 @@ string IntegerArgument::subprintlong() const {
 		str += _("<=");
 		str += " ";
 		str += imax->print();
+	} else if(i_inttype != INTEGER_TYPE_NONE) {
+		str += " ";
+		str += _("and");
+		str += " ";
+		str += _("<=");
+		str += " ";
+		switch(i_inttype) {
+			case INTEGER_TYPE_SIZE: {}
+			case INTEGER_TYPE_UINT: {str += i2s(UINT_MAX); break;}
+			case INTEGER_TYPE_SINT: {str += i2s(INT_MAX); break;}
+			case INTEGER_TYPE_ULONG: {str += i2s(ULONG_MAX); break;}
+			case INTEGER_TYPE_SLONG: {str += i2s(LONG_MAX); break;}
+			default: {}
+		}
 	}
 	return str;
 }
@@ -1746,17 +1803,24 @@ bool TextArgument::suggestsQuotes() const {return false;}
 DateArgument::DateArgument(string name_, bool does_test, bool does_error) : Argument(name_, does_test, does_error) {b_text = true;}
 DateArgument::DateArgument(const DateArgument *arg) {set(arg); b_text = true;}
 DateArgument::~DateArgument() {}
+void DateArgument::parse(MathStructure *mstruct, const string &str, const ParseOptions &po) const {
+	if(b_text && str.find_first_of(PARENTHESISS, 1) != string::npos) {
+		CALCULATOR->parse(mstruct, str, po);
+	} else {
+		Argument::parse(mstruct, str, po);
+	}
+}
 bool DateArgument::subtest(MathStructure &value, const EvaluationOptions &eo) const {
 	if(!value.isSymbolic()) {
 		value.eval(eo);
 	}
-	int day = 0, year = 0, month = 0;
-	return value.isSymbolic() && s2date(value.symbol(), day, year, month);
+	QalculateDate date;
+	return value.isSymbolic() && date.set(value.symbol());
 }
 int DateArgument::type() const {return ARGUMENT_TYPE_DATE;}
 Argument *DateArgument::copy() const {return new DateArgument(this);}
-string DateArgument::print() const {return _("date");}
-string DateArgument::subprintlong() const {return _("a date");}
+string DateArgument::print() const {return string(_("date")) + " (Y-M-D)";}
+string DateArgument::subprintlong() const {return string(_("a date")) + " (Y-M-D)";}
 
 VectorArgument::VectorArgument(string name_, bool does_test, bool allow_matrix, bool does_error) : Argument(name_, does_test, does_error) {
 	setMatrixAllowed(allow_matrix);
@@ -1781,7 +1845,10 @@ bool VectorArgument::subtest(MathStructure &value, const EvaluationOptions &eo)
 	//if(!value.isVector()) {
 		value.eval(eo);
 	//}
-	if(!value.isVector()) return false;
+	if(!value.isVector()) {
+		if(isLastArgument()) value.transform(STRUCT_VECTOR);
+		else return false;
+	}
 	if(b_argloop && subargs.size() > 0) {
 		for(size_t i = 0; i < value.countChildren(); i++) {
 			if(!subargs[i % subargs.size()]->test(value[i], 1, NULL, eo)) {
