@@ -1,7 +1,7 @@
 /*
     Qalculate (libary)
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -220,13 +220,19 @@ AliasUnit::AliasUnit(string cat_, string name_, string plural_, string short_nam
 	remove_blank_ends(inverse);
 	svalue = relation;
 	sinverse = inverse;
+	suncertainty = "";
 	i_exp = exp;
+	i_mix = 0;
+	i_mix_min = 0;
 }
 AliasUnit::AliasUnit() {
 	o_unit = NULL;
 	svalue = "";
 	sinverse = "";
+	suncertainty = "";
 	i_exp = 1;
+	i_mix = 0;
+	i_mix_min = 0;
 }
 AliasUnit::AliasUnit(const AliasUnit *unit) {
 	set(unit);
@@ -244,6 +250,9 @@ void AliasUnit::set(const ExpressionItem *item) {
 			i_exp = u->firstBaseExponent();
 			svalue = u->expression();
 			sinverse = u->inverseExpression();
+			suncertainty = u->uncertainty();
+			i_mix = u->mixWithBase();
+			i_mix_min = u->mixWithBaseMinimum();
 		}
 	} else {
 		ExpressionItem::set(item);
@@ -265,6 +274,9 @@ string AliasUnit::expression() const {
 string AliasUnit::inverseExpression() const {
 	return sinverse;
 }
+string AliasUnit::uncertainty() const {
+	return suncertainty;
+}
 void AliasUnit::setExpression(string relation) {
 	remove_blank_ends(relation);
 	if(relation.empty()) {
@@ -279,6 +291,12 @@ void AliasUnit::setInverseExpression(string inverse) {
 	sinverse = inverse;
 	setChanged(true);
 }
+void AliasUnit::setUncertainty(string standard_uncertainty) {
+	remove_blank_ends(standard_uncertainty);
+	suncertainty = standard_uncertainty;
+	if(!suncertainty.empty()) setApproximate(true);
+	setChanged(true);
+}
 MathStructure &AliasUnit::convertToBaseUnit(MathStructure &mvalue, MathStructure &mexp) const {
 	convertToFirstBaseUnit(mvalue, mexp);
 	return o_unit->convertToBaseUnit(mvalue, mexp);
@@ -327,8 +345,9 @@ int AliasUnit::baseExponent(int exp) const {
 MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathStructure &mexp) const {
 	if(i_exp != 1) mexp /= i_exp;
 	ParseOptions po;
-	if(isApproximate() && precision() < 1) {
-		po.read_precision = ALWAYS_READ_PRECISION;
+	if(isApproximate() && suncertainty.empty() && precision() == -1) {
+		if(sinverse.find(DOT) || svalue.find(DOT)) po.read_precision = READ_PRECISION_WHEN_DECIMALS;
+		else po.read_precision = ALWAYS_READ_PRECISION;
 	}
 	if(sinverse.empty()) {
 		if(svalue.find("\\x") != string::npos) {
@@ -346,24 +365,36 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 			CALCULATOR->parse(&mvalue, stmp, po);
 			CALCULATOR->delId(x_id);
 			CALCULATOR->delId(y_id);
-			if(precision() > 0 && (mvalue.precision() < 1 || precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
+			if(precision() > 0 && (mvalue.precision() < 0 || precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
 			if(isApproximate()) mvalue.setApproximate(true, true);
 		} else {
 			MathStructure *mstruct = new MathStructure();
-			CALCULATOR->parse(mstruct, svalue, po);
-			if(precision() > 0) {
+			if(suncertainty.empty()) {
+				CALCULATOR->parse(mstruct, svalue, po);
+			} else {
+				mstruct->number().set(svalue, po);
+				mstruct->numberUpdated();
+			}
+			if(!suncertainty.empty()) {
+				Number nr_u(suncertainty);
+				if(mstruct->isNumber()) {
+					mstruct->number().setUncertainty(nr_u);
+				} else if(mstruct->isMultiplication() && mstruct->size() > 0 && (*mstruct)[0].isNumber()) {
+					(*mstruct)[0].number().setUncertainty(nr_u);
+				}
+			} else if(precision() > 0) {
 				if(mstruct->isNumber()) {
 					if(mstruct->number().precision() < 1 || precision() < mstruct->number().precision()) {
 						mstruct->number().setPrecision(precision());
 						mstruct->numberUpdated();
 					}
 				} else if(mstruct->isMultiplication() && mstruct->getChild(1)->isNumber()) {
-					if(mstruct->getChild(1)->number().precision() < 1 || precision() < mstruct->getChild(1)->number().precision()) {
+					if(mstruct->getChild(1)->number().precision() < 0 || precision() < mstruct->getChild(1)->number().precision()) {
 						mstruct->getChild(1)->number().setPrecision(precision());
 						mstruct->getChild(1)->numberUpdated();
 						mstruct->childUpdated(1);
 					}
-				} else if(mstruct->precision() < 1 || precision() < mstruct->precision()) {
+				} else if(mstruct->precision() < 0 || precision() < mstruct->precision()) {
 					mstruct->setPrecision(precision(), true);
 				}
 			} else if(isApproximate() && !mstruct->isApproximate()) {
@@ -388,24 +419,24 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 			CALCULATOR->parse(&mvalue, stmp, po);
 			CALCULATOR->delId(x_id);
 			CALCULATOR->delId(y_id);
-			if(precision() > 0 && (mvalue.precision() < 1 || precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
+			if(precision() > 0 && (mvalue.precision() < 0|| precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
 			if(isApproximate()) mvalue.setApproximate(true, true);
 		} else {
 			MathStructure *mstruct = new MathStructure();
 			CALCULATOR->parse(mstruct, sinverse, po);
 			if(precision() > 0) {
 				if(mstruct->isNumber()) {
-					if(mstruct->number().precision() < 1 || precision() < mstruct->number().precision()) {
+					if(mstruct->number().precision() < 0 || precision() < mstruct->number().precision()) {
 						mstruct->number().setPrecision(precision());
 						mstruct->numberUpdated();
 					}
 				} else if(mstruct->isMultiplication() && mstruct->getChild(1)->isNumber()) {
-					if(mstruct->getChild(1)->number().precision() < 1 || precision() < mstruct->getChild(1)->number().precision()) {
+					if(mstruct->getChild(1)->number().precision() < 0 || precision() < mstruct->getChild(1)->number().precision()) {
 						mstruct->getChild(1)->number().setPrecision(precision());
 						mstruct->getChild(1)->numberUpdated();
 						mstruct->childUpdated(1);
 					}
-				} else if(mstruct->precision() < 1 || precision() < mstruct->precision()) {
+				} else if(mstruct->precision() < 0 || precision() < mstruct->precision()) {
 					mstruct->setPrecision(precision(), true);
 				}
 			} else if(isApproximate() && !mstruct->isApproximate()) {
@@ -419,8 +450,9 @@ MathStructure &AliasUnit::convertFromFirstBaseUnit(MathStructure &mvalue, MathSt
 }
 MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStructure &mexp) const {
 	ParseOptions po;
-	if(isApproximate() && precision() < 1) {
-		po.read_precision = ALWAYS_READ_PRECISION;
+	if(isApproximate() && suncertainty.empty() && precision() == -1) {
+		if(svalue.find(DOT)) po.read_precision = READ_PRECISION_WHEN_DECIMALS;
+		else po.read_precision = ALWAYS_READ_PRECISION;
 	}
 	if(svalue.find("\\x") != string::npos) {
 		string stmp = svalue;
@@ -444,24 +476,36 @@ MathStructure &AliasUnit::convertToFirstBaseUnit(MathStructure &mvalue, MathStru
 		} else {
 			CALCULATOR->delId(y_id);
 		}
-		if(precision() > 0 && (mvalue.precision() < 1 || precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
+		if(precision() >= 0 && (mvalue.precision() < 0 || precision() < mvalue.precision())) mvalue.setPrecision(precision(), true);
 		if(isApproximate()) mvalue.setApproximate(true, true);
 	} else {
 		MathStructure *mstruct = new MathStructure();
-		CALCULATOR->parse(mstruct, svalue, po);
-		if(precision() > 0) {
+		if(suncertainty.empty()) {
+			CALCULATOR->parse(mstruct, svalue, po);
+		} else {
+			mstruct->number().set(svalue, po);
+			mstruct->numberUpdated();
+		}
+		if(!suncertainty.empty()) {
+			Number nr_u(suncertainty);
 			if(mstruct->isNumber()) {
-				if(mstruct->number().precision() < 1 || precision() < mstruct->number().precision()) {
+				mstruct->number().setUncertainty(nr_u);
+			} else if(mstruct->isMultiplication() && mstruct->size() > 0 && (*mstruct)[0].isNumber()) {
+				(*mstruct)[0].number().setUncertainty(nr_u);
+			}
+		} else if(precision() >= 0) {
+			if(mstruct->isNumber()) {
+				if(mstruct->number().precision() < 0 || precision() < mstruct->number().precision()) {
 					mstruct->number().setPrecision(precision());
 					mstruct->numberUpdated();
 				}
 			} else if(mstruct->isMultiplication() && mstruct->getChild(1)->isNumber()) {
-				if(mstruct->getChild(1)->number().precision() < 1 || precision() < mstruct->getChild(1)->number().precision()) {
+				if(mstruct->getChild(1)->number().precision() < 0 || precision() < mstruct->getChild(1)->number().precision()) {
 					mstruct->getChild(1)->number().setPrecision(precision());
 					mstruct->getChild(1)->numberUpdated();
 					mstruct->childUpdated(1);
 				}
-			} else if(mstruct->precision() < 1 || precision() < mstruct->precision()) {
+			} else if(mstruct->precision() < 0 || precision() < mstruct->precision()) {
 				mstruct->setPrecision(precision(), true);
 			}
 		} else if(isApproximate() && !mstruct->isApproximate()) {
@@ -480,6 +524,10 @@ void AliasUnit::setExponent(int exp) {
 int AliasUnit::firstBaseExponent() const {
 	return i_exp;
 }
+int AliasUnit::mixWithBase() const {return i_mix;}
+int AliasUnit::mixWithBaseMinimum() const {return i_mix_min;}
+void AliasUnit::setMixWithBase(int mix_priority) {i_mix = mix_priority;}
+void AliasUnit::setMixWithBaseMinimum(int mix_minimum) {i_mix_min = mix_minimum;}
 int AliasUnit::subtype() const {
 	return SUBTYPE_ALIAS_UNIT;
 }
@@ -809,11 +857,11 @@ void CompositeUnit::setBaseExpression(string base_expression_) {
 	eo.keep_prefixes = true;
 	eo.structuring = STRUCTURING_NONE;
 	eo.reduce_divisions = false;
+	eo.do_polynomial_division = false;
 	ParseOptions po;
 	po.variables_enabled = false;
 	po.functions_enabled = false;
 	po.unknowns_enabled = true;
-	po.convert_temperature_units = false;
 	MathStructure mstruct;
 	bool had_errors = false;
 	CALCULATOR->beginTemporaryStopMessages();
