@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -167,8 +167,8 @@ class AliasUnit : public Unit {
 
   protected:
 
-	string svalue, sinverse;
-	int i_exp;
+	string svalue, sinverse, suncertainty;
+	int i_exp, i_mix, i_mix_min;
 	Unit *o_unit;
 
   public:
@@ -186,6 +186,7 @@ class AliasUnit : public Unit {
 	virtual void setBaseUnit(Unit *alias);
 	virtual string expression() const;
 	virtual string inverseExpression() const;
+	virtual string uncertainty() const;
 	/**
 	* Sets the relation expression.
 	*/
@@ -194,6 +195,7 @@ class AliasUnit : public Unit {
 	* Sets the inverse relation expression.
 	*/
 	virtual void setInverseExpression(string inverse);
+	virtual void setUncertainty(string standard_uncertainty);
 	virtual MathStructure &convertToFirstBaseUnit(MathStructure &mvalue, MathStructure &mexp) const;
 	virtual MathStructure &convertFromFirstBaseUnit(MathStructure &mvalue, MathStructure &mexp) const;
 	virtual MathStructure &convertToBaseUnit(MathStructure &mvalue, MathStructure &mexp) const;
@@ -205,6 +207,10 @@ class AliasUnit : public Unit {
 	virtual int baseExponent(int exp = 1) const;
 	virtual void setExponent(int exp);
 	virtual int firstBaseExponent() const;
+	virtual int mixWithBase() const;
+	virtual int mixWithBaseMinimum() const;
+	virtual void setMixWithBase(int combine_priority = 1);
+	virtual void setMixWithBaseMinimum(int combine_minimum);
 	virtual int subtype() const;
 	virtual bool isChildOf(Unit *u) const;
 	virtual bool isParentOf(Unit *u) const;
