@@ -1,7 +1,7 @@
 /*
     Qalculate (library)
 
-    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna_k@fmgirl.com)
+    Copyright (C) 2003-2007, 2008, 2016  Hanna Knutsson (hanna.knutsson@protonmail.com)
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -31,8 +31,8 @@
 * Each type is a subset of the type above.
 */
 typedef enum {
+	/// Multiplication is NOT commutative; do not use
 	ASSUMPTION_TYPE_NONE = 0,
-	/// Multiplication is NOT commutative
 	ASSUMPTION_TYPE_NONMATRIX = 1,
 	ASSUMPTION_TYPE_NUMBER = 2,
 	/// im(x) != 0
@@ -98,6 +98,7 @@ class Assumptions {
 	bool isComplex();
 	bool isNonZero();
 	bool isNonMatrix();
+	bool isScalar();
 	
 	AssumptionType type();
 	AssumptionSign sign();
@@ -147,6 +148,8 @@ class Variable : public ExpressionItem {
 	virtual bool representsNonNegative(bool = false) {return false;}
 	virtual bool representsNonPositive(bool = false) {return false;}
 	virtual bool representsInteger(bool = false) {return false;}
+	virtual bool representsNonInteger(bool = false) {return false;}
+	virtual bool representsFraction(bool = false) {return false;}
 	virtual bool representsNumber(bool = false) {return false;}
 	virtual bool representsRational(bool = false) {return false;}
 	virtual bool representsReal(bool = false) {return false;}
@@ -157,6 +160,7 @@ class Variable : public ExpressionItem {
 	virtual bool representsUndefined(bool = false, bool = false, bool = false) {return false;}
 	virtual bool representsBoolean() {return false;}
 	virtual bool representsNonMatrix() {return false;}
+	virtual bool representsScalar() {return false;}
 	
 };
 
@@ -217,6 +221,7 @@ class UnknownVariable : public Variable {
 	virtual bool representsComplex(bool = false);
 	virtual bool representsNonZero(bool = false);
 	virtual bool representsNonMatrix();
+	virtual bool representsScalar();
 	
 };
 
@@ -233,7 +238,8 @@ class KnownVariable : public Variable {
 	MathStructure *mstruct;
 	bool b_expression;
  	int calculated_precision;
-	string sexpression;
+ 	bool calculated_with_interval, calculated_with_units;
+	string sexpression, suncertainty, sunit;
 
   public:
   
@@ -282,6 +288,9 @@ class KnownVariable : public Variable {
 	* @returns The variable's expression.
 	*/
 	virtual string expression() const;
+	virtual string uncertainty() const;
+	virtual string unit() const;
+	
 	int subtype() const {return SUBTYPE_KNOWN_VARIABLE;}
 
 	/** Sets the value of the variable. If expression is set, it is cleared.
@@ -293,7 +302,9 @@ class KnownVariable : public Variable {
 	*
 	* @param expression_ Expression.
 	*/
-	virtual void set(string expression_);	
+	virtual void set(string expression_);
+	virtual void setUncertainty(string standard_uncertainty);
+	virtual void setUnit(string unit_expression);
 
 	/** Returns the value of the variable. If no value is set or parsed and an expression is set, the expression is parsed and resulting value returned.
 	*
@@ -306,6 +317,8 @@ class KnownVariable : public Variable {
 	virtual bool representsNonNegative(bool = false);
 	virtual bool representsNonPositive(bool = false);
 	virtual bool representsInteger(bool = false);
+	virtual bool representsNonInteger(bool = false);
+	virtual bool representsFraction(bool = false);
 	virtual bool representsNumber(bool = false);
 	virtual bool representsRational(bool = false);
 	virtual bool representsReal(bool = false);
@@ -316,6 +329,7 @@ class KnownVariable : public Variable {
 	virtual bool representsUndefined(bool = false, bool = false, bool = false);
 	virtual bool representsBoolean();
 	virtual bool representsNonMatrix();
+	virtual bool representsScalar();
 
 };
 
@@ -348,12 +362,14 @@ class DynamicVariable : public KnownVariable {
 	* @returns Precision of the calculated value or zero if the value has not yet been calculated.
 	*/
 	int calculatedPrecision() const;
+	bool calculatedWithInterval() const;
 	
 	virtual bool representsPositive(bool = false) {return true;}
 	virtual bool representsNegative(bool = false) {return false;}
 	virtual bool representsNonNegative(bool = false) {return true;}
 	virtual bool representsNonPositive(bool = false) {return false;}
 	virtual bool representsInteger(bool = false) {return false;}
+	virtual bool representsNonInteger(bool = false) {return true;}
 	virtual bool representsNumber(bool = false) {return true;}
 	virtual bool representsRational(bool = false) {return false;}
 	virtual bool representsReal(bool = false) {return true;}
@@ -364,6 +380,7 @@ class DynamicVariable : public KnownVariable {
 	virtual bool representsUndefined(bool = false, bool = false, bool = false) {return false;}
 	virtual bool representsBoolean() {return false;}
 	virtual bool representsNonMatrix() {return true;}
+	virtual bool representsScalar() {return true;}
 
 };
 
@@ -375,6 +392,16 @@ DECLARE_BUILTIN_VARIABLE(EVariable)
 DECLARE_BUILTIN_VARIABLE(EulerVariable)
 /// Dynamic variable for Catalan's constant
 DECLARE_BUILTIN_VARIABLE(CatalanVariable)
-
+/// Dynamic variable for current precision
+class PrecisionVariable : public DynamicVariable {
+  private:
+	void calculate() const;
+  public: \
+	PrecisionVariable();
+	PrecisionVariable(const PrecisionVariable *variable) {set(variable);}
+	ExpressionItem *copy() const {return new PrecisionVariable(this);}
+	bool representsInteger(bool = false) {return true;}
+	bool representsNonInteger(bool = false) {return false;}
+};
 
 #endif
